package com.app.web.jsf.component;

import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.faces.component.UIComponentBase;
import javax.faces.context.FacesContext;
import javax.faces.el.ValueBinding;



public class FormBeanComponent extends UIComponentBase {

	public static final String COMPONENT_TYPE = FormBeanComponent.class.getName();
	public static final String DEFAULT_RENDERER_TYPE = HtmlFormBeanRenderer.class.getName();

	/**
	 * En este atributo se guarda la instancia de la clase que se va a pintar en el formulario y donde se
	 * guardarán los valores introducidos por el usuario.
	 * El tipo es <tt>ValueBinding</tt> porque acepta EL de JSF.
	 */
	private ValueBinding beanBinding = null;
	
	/**
	 * Este atributo ha de guardar la información extraida de la request hasta que se alcance la fase de update model.
	 * Será entonces donde los valores almacenados en el componente se usarán para actualizar la información del Back Bean.
	 */
	private Map<String, String> localValues = new HashMap<String, String>();
	
	
	public FormBeanComponent() {
		setRendererType(DEFAULT_RENDERER_TYPE);
	}
	
	/**
	 * Inidca el nombre de la familia del componente.
	 * Este nombre se usará para determinar el render que se tiene que usar para pintar este componente.
	 */
	@Override
	public String getFamily() {
		return COMPONENT_TYPE;
	}

	/**
	 * @return the bean
	 */
	public Object getBean() {
		final Object bean = beanBinding.getValue(FacesContext.getCurrentInstance());
		return bean;
	}

	
	/**
	 * @param bean the bean to set
	 */
	public void setBean(Object bean) {
		beanBinding.setValue(FacesContext.getCurrentInstance(), bean);
	}
	
	/**
	 * @return the beanBinding
	 */
	public ValueBinding getBeanBinding() {
		return beanBinding;
	}

	
	/**
	 * @param beanBinding the beanBinding to set
	 */
	public void setBeanBinding(ValueBinding beanBinding) {
		this.beanBinding = beanBinding;
	}

	/* (non-Javadoc)
	 * @see javax.faces.component.UIComponentBase#restoreState(javax.faces.context.FacesContext, java.lang.Object)
	 */
	@Override
	public void restoreState(FacesContext context, Object state) {
		final Object values[] = (Object[])state;
		super.restoreState(context, values[0]);
		beanBinding = (ValueBinding)restoreAttachedState(FacesContext.getCurrentInstance(), values[1]);
	}

	/* (non-Javadoc)
	 * @see javax.faces.component.UIComponentBase#saveState(javax.faces.context.FacesContext)
	 */
	@Override
	public Object saveState(FacesContext context) {
		List<Object> state = new ArrayList<Object>();
		state.add( super.saveState(context) );
		state.add( saveAttachedState( FacesContext.getCurrentInstance(), beanBinding ) );
		return state.toArray();
	}

	
	
	/**
	 * Se llama en la fase de Update Model Values para actualizar los valores del Back Bean con los valores del componente.
	 *  
	 * @see javax.faces.component.UIComponentBase#processUpdates(javax.faces.context.FacesContext)
	 */
	@Override
	public void processUpdates(FacesContext facesContext) {
		final Object bean = getBean();
		final Class<?> clazz = bean.getClass();
		final Class[] parameterTypes = { String.class };

		for (Map.Entry newValue : localValues.entrySet()) {
			final String key = newValue.getKey().toString();
			final String setterName = "set" + key.substring(0, 1).toUpperCase() + key.substring(1);
			Method setterMethod;
			try {
				final Object[] setterArguments = { newValue.getValue() };
				setterMethod = clazz.getMethod(setterName, parameterTypes );
				setterMethod.invoke(bean, setterArguments);
				
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}

	/**
	 * @return the localValues
	 */
	public Map<String, String> getLocalValues() {
		return localValues;
	}

	
	/**
	 * @param localValues the localValues to set
	 */
	public void setLocalValues(Map<String, String> localValue) {
		this.localValues = localValue;
	}
}
