package com.app.web.jsf.utils;

import java.util.Locale;
import java.util.Map;

import javax.faces.application.Application;
import javax.faces.application.FacesMessage;
import javax.faces.context.FacesContext;

import org.apache.commons.logging.Log;
import org.apache.myfaces.shared_impl.util.MessageUtils;

/**
 * Clase de utilidad para facilitar el trabajo con JSF.
 *
 * <p>Todos los métoso son estáticos y no se puedan crear instancias.
 */
public final class JsfUtils {

	/**
	 * Nadie debe construir instancias de esta clase
	 */
	private JsfUtils() {
		// Nadie debe construir instancias de esta clase
	}

	/**
	 * Devuelve el mapa de parámetros asociados a la request del faces context asociado al thread.
	 * @return mapa de atributos asociados a la request del faces context asociado al thread.
	 */
	@SuppressWarnings("unchecked")
	public static Map<String, Object> getRequestParametersMap() {
		final Map<String, Object> map = FacesContext.getCurrentInstance().getExternalContext().getRequestParameterMap();
		return map;
	}

	/**
	 * Devuelve el mapa de atributos asociados a la request del faces context que se pasa como parámetro.
	 * @return mapa de atributos asociados a la request del faces context que se pasa como parámetro.
	 */
	@SuppressWarnings("unchecked")
	public static Map<String, Object> getRequestMap(FacesContext facesContext) {
		final Map<String, Object> map = facesContext.getExternalContext().getRequestMap();
		return map;
	}

	/**
	 * Devuelve el mapa de atributos asociados a la request del faces context asociado al thread.
	 * @return mapa de atributos asociados a la request del faces context asociado al thread.
	 */
	@SuppressWarnings("unchecked")
	public static Map<String, Object> getRequestMap() {
		final Map<String, Object> map = getRequestMap(FacesContext.getCurrentInstance());
		return map;
	}

	/**
	 * Devuelve el mapa de atributos asociados a la session del faces context asociado al thread.
	 * @return mapa de atributos asociados a la session del faces context asociado al thread.
	 */
	@SuppressWarnings("unchecked")
	public static Map<String, Object> getSessionMap() {
		final Map<String, Object> map = FacesContext.getCurrentInstance().getExternalContext().getSessionMap();
		return map;
	}

	/**
	 * Devuelve el mapa de atributos asociados al application del faces context asociado al thread.
	 * @return mapa de atributos asociados al application del faces context asociado al thread.
	 */
	@SuppressWarnings("unchecked")
	public static Map<String, Object> getApplicationMap() {
		final Map<String, Object> map = FacesContext.getCurrentInstance().getExternalContext().getApplicationMap();
		return map;
	}

	/**
	 * Dado el id de un mensaje de error, mete el mensaje de jsf traducido según el locale del usuario.
	 * Y deja una traza de error traduciendo el id según el locale de la máquina virtual.
	 *
	 * @param errId identificador del mensaje de error.
	 * @param log log donde se va a dejar la traza.
	 * @param e excepción para añadir al log.
	 */
	public static void sendErrorMsgToUserAndLog(String errId, Log log, Exception e) {
		final FacesContext context = FacesContext.getCurrentInstance();
    	final FacesMessage jsfErrMsg = MessageUtils.getMessage(context, errId);
    	final FacesMessage logErrMsg = MessageUtils.getMessage(Locale.getDefault(), errId, null);
		log.error(logErrMsg.getDetail(), e);
		context.addMessage(null, jsfErrMsg);
	}
}
